package Finance::CoinbasePro::API::CLI::ConversionFactor;  # stores a single currency->currency conversion factor
use Mouse;

has 'from_currency'  => ( is=>'rw', isa=>"Str", required=>1 );
has 'to_currency'    => ( is=>'rw', isa=>"Str", required=>1 );
has 'factor'         => ( is=>'rw', isa=>"Num", required=>1 );

1;









#################### pod generated by Pod::Autopod - keep this line to make pod updates possible ####################

=head1 NAME

Finance::CoinbasePro::API::CLI::ConversionFactor - stores a single currency->currency conversion factor


=head1 DESCRIPTION

pod generated by Pod::Autopod - keep this line to make pod updates possible ####################


=head1 REQUIRES

L<Mouse> 


=head1 METHODS


=cut

